require 'elasticsearch/dsl'
require 'pry-remote'
module PdfSearch
  class ElasticSearchQuery
    include Elasticsearch::DSL
    attr_reader :client_query_specification, :search_index
    
    def initialize(client_query_specification, search_index)
      @search_index =  search_index
      @client_query_specification = client_query_specification
    end

    def to_hash
      {
        "query": {
          "bool": {
            "must": [
              {
                "match": {
                  "text": client_query_specification['search']
                }
              }
            ].concat(range_queries)
          }
        }
      }
    end

    def range_queries
      return [] if search_index.search_input_fields_by_type.nil?
      search_index.search_input_fields_by_type[:interval].map do |name|
        {
          "range": {
            name => {
              gte: client_query_specification["search_#{name}_start"],
              lte: client_query_specification["search_#{name}_end"]
            }
          }
        }
      end
    end
  end
end