require 'webrick'
require 'json'

class PdfSearch::WebServer
  attr_accessor :daemon

  def initialize(search_index, debug_mode = false)
    @search_index = search_index
    @debug_mode = (ENV['DEBUG_PDF_SEARCH'] == '1') || debug_mode
  end

  def basic_auth
    return @basic_auth if @basic_auth != nil

    config = { :Realm => 'BanalBI Pdf-Search' }

    htpasswd = WEBrick::HTTPAuth::Htpasswd.new 'banalbi-pdf-search'
    htpasswd.set_passwd config[:Realm], ENV['PDF_SEARCH_USERNAME'] , ENV['PDF_SEARCH_PASSWORD']
    htpasswd.flush

    config[:UserDB] = htpasswd

    @basic_auth = WEBrick::HTTPAuth::BasicAuth.new config
  end

  def index(request, response)
    search_view = ::PdfSearch::SearchView.new(@search_index)
    response.status = 200
    response['Content-Type'] = 'text/html'
    response.body = search_view.render
  end

  def auth_defined?
    [ENV['PDF_SEARCH_USERNAME'], ENV['PDF_SEARCH_PASSWORD']].all? do |env_var|
      env_var != nil && env_var != ''
    end
  end

  def search(request, response)
    if auth_defined?
      basic_auth.authenticate(request, response)
    end

    query = ::PdfSearch::ElasticSearchQuery.new(JSON.parse(request.body), @search_index)
    elastic_response = ::PdfSearch::ElasticSearchClient.search index: 'pdf_pages', body: query.to_hash

    response.body = response_html(elastic_response)
  end

  def start
    start_server = lambda do
      server = WEBrick::HTTPServer.new(:Port => 80)

      server.mount_proc '/', &method(:index)
      server.mount_proc '/search', &method(:search)

      server.start
    end

    if @debug_mode
      start_server.call
    else
	    self.daemon = Daemons.call(multiple: true, &start_server)
    end
  end

  def response_html(es_response)
    results = es_response['hits']['hits']
    results.map do |result|
      "<li>#{result["_source"]["text"]}</li>"
    end.join("\n")
  end
end
